/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 2 Exercitiul 3
 * Digital I/O
 *********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>
#undef F_CPU
#include <string.h>
#include "piano.h"

/* Current working mode */
mode crt_mode;

/* Time variable used for recording and playing a recorded song */
unsigned int start_time;
/* The placeholder for a recorded song */
note song[MAX_SONG_SIZE];
/* The length (int notes) of the recorded song */
int song_size;
/* Index of the current note, used for recording and playing */
int crt_note;
/* The current duration of the current note */
unsigned int crt_delay;

/* Variables used for debouncing */
unsigned int control;
unsigned int keyboard;
/* Active notes, used for handling the concurrent pressing of two
 * or more keynotes */
unsigned int active_keyboard;


/* Piano logic; called every 10 ms */
ISR(TIMER2_COMP_vect) 
{
        /* The index of a pressed keybutton */
        int index = -1;
        int i;

        /* Test current mode */
        if (crt_mode != PLAY) {
                /* REC or SING*/
                /* Check pressed buttons for keyboard */
                for (i = 0; i < 8; ++i)
                        if (!(PINA & _BV(i))) {
                                /* The button was pressed 10 ms so we mark it
                                 * as active (debouncing)*/
                                if ((keyboard & _BV(i)) > 0) {
                                        setNote(i);
                                        setPWM();
                                        active_keyboard |= _BV(i);
                                        keyboard &= _BV(i);
                                        index = i;
                                } else {
                                        keyboard |= _BV(i);
                                        active_keyboard &= ~_BV(i);
                                }
                        } else {
                                active_keyboard &= ~_BV(i);
                                /* If no other key is active we close the
                                 * output */
                                if ((active_keyboard == 0) && (PINA & _BV(i))) 
                                        unsetPWM();

                        }
        } else {
                /* mode: PLAY */
                 if (crt_delay + 1 < song[crt_note].delay) {
                        if (song[crt_note].index == -1) {
                                /* If the current note is mute we close the
                                 * output */
                                unsetPWM();
                        } else {
                                /* We set the current note frequency and we
                                 * open the output */
                                setNote(song[crt_note].index);
                                setPWM();
                        }
                        crt_delay ++;
                 } else {
                        crt_delay = 0;
                        crt_note ++;
                        /* Test for the enf of song */
                        if (crt_note >= song_size) {
                                unsetPWM();
                                /* Repeat song */
                                crt_note = 0;
                                _delay_ms(300);
                        }
                 }
        
        }

        if (crt_mode == REC) {
                /* We increase the last note's duration */
                if (song[crt_note].index == index) {
                        song[crt_note].delay += 1;
                } else {
                        /* We register a new note */
                        crt_note = (crt_note + 1) % MAX_SONG_SIZE;
                        song[crt_note].index = index;
                        song[crt_note].delay += 1;
                        song_size = crt_note + 1;
                }
        }       

        /* Check pressed buttons for control */
        /* REC button */
        if (!(PINC & _BV(B_REC)) ) {
                if ((control & _BV(B_REC))) {
                        PORTC &= ~_BV(L_PLAY);
                        PORTC |= _BV(L_REC);
                        crt_mode = REC;
                        control &= ~_BV(B_REC);
                        crt_note = 0;
                        start_time = 0;
                        crt_delay = 0;
                        song_size = 0;
                        memset(song, 0, MAX_SONG_SIZE * sizeof(note));
                } else {
                        control |= _BV(B_REC);
                }

        }
        /* PLAY button */
        if (!(PINC & _BV(B_PLAY)) ) {
                if ((control & _BV(B_PLAY))) {
                        PORTC &= ~_BV(L_REC);
                        PORTC |= _BV(L_PLAY);
                        crt_mode = PLAY;
                        control &= ~_BV(B_PLAY);
                        crt_note = 0;
                        crt_delay = 0;
                } else {
                        control |= _BV(B_PLAY);
                }

        }
        /* SING button */
        if (!(PINC & _BV(B_SING)) ) {
                if ((control & _BV(B_SING))) {
                        PORTC &= ~_BV(L_PLAY);
                        PORTC &= ~_BV(L_REC);
                        crt_mode = SING;
                        control &= ~_BV(B_SING);
                } else {
                        control |= _BV(B_SING);
                }

        }
}


void  initBoard()
{
        /* Set the output pin for buzzer*/
        DDRD = _BV(PD5);		
        TCCR1A|=_BV(WGM10)|_BV(COM1A0); 
        TCCR1B|=_BV(WGM13)|_BV(CS12);

        /* Set the keyboard buttons */
        DDRA = 0;                     
        PORTA = 0xff;                

        /* Set the command pannel */
        DDRC = _BV(7) | _BV(6);     
        PORTC = 0xff - _BV(7) - _BV(6);

        /* Initialize mode */
        crt_mode = SING;

        /* Initialize outer board */
        /* Button 6*/
	DDRD &= ~_BV(6);
	PORTD |= _BV(6);
        /* Led 7 */
        PORTD &= ~_BV(7);
}



int main()
{
	srand(1);
	TIMSK = 0;
	
	
	sei();

        initBoard();
        initTimer();
	 
        while(1)
        {
        /* Wait for interrupts */
        }  
	 
	return 0;
}
